class TimeManager {
  constructor() {
    this.tasks = new Map(); // Store tasks with their ids
    this.timer = null; // Main timer
    this.precision = 100; // Timer precision
  }

  // Generate a unique id for tasks
  _generateId() {
    return Date.now().toString(36) + Math.random().toString(36).substr(2, 5);
  }

  // Format time in hours, minutes, and seconds
  _formatTime(ms) {
    const hours = Math.floor(ms / 3600000);
    const minutes = Math.floor((ms % 3600000) / 60000);
    const seconds = Math.floor((ms % 60000) / 1000);
    return `${hours}小时${minutes}分${seconds}秒`;
  }

  // Start the main timer
  _startTimer() {
    if (!this.timer) {
      this.timer = setInterval(() => {
        this._updateTasks();
      }, this.precision);
    }
  }

  // Update tasks based on their type and status
  _updateTasks() {
    for (const [id, task] of this.tasks.entries()) {
      if (task.status === 'running') {
        if (task.type === 'stopwatch') {
          task.elapsed += this.precision;
          task.updateHook(id, this._formatTime(task.elapsed));
        } else if (task.type === 'countdown') {
          task.remaining -= this.precision;
          task.updateHook(id, this._formatTime(task.remaining));
          if (task.remaining <= 0) {
            task.endHook(id);
            this.tasks.delete(id);
          }
        }
      }
    }
  }

  // Add a stopwatch task
  addStopwatch(updateHook) {
    const id = this._generateId();
    this.tasks.set(id, {
      type: 'stopwatch',
      status: 'running',
      elapsed: 0,
      updateHook: updateHook,
    });
    this._startTimer();
    return id;
  }

  // Add a countdown task
  addCountdown(duration, updateHook, endHook) {
      const id = this._generateId();
      this.tasks.set(id, {
        type: 'countdown',
        status: 'running',
        remaining: duration,
        updateHook: updateHook,
        endHook: endHook,
      });
      this._startTimer(); // 删除重复的调用
      return id;
    }


  // 继续任务
  continueTask(id) {
    const task = this.tasks.get(id);
    if (task && task.status === 'paused') {
      task.status = 'running';
    }
  }
// Pause a task
  pauseTask(id) {
    const task = this.tasks.get(id);
    if (task && task.status === 'running') {
      task.status = 'paused';
    }
  }

  // Remove a task
  removeTask(id) {
    this.tasks.delete(id);
  }

}